%namespace GPPGParserScanner

%using PascalABCCompiler.Oberon00Parser;
%using PascalABCCompiler.SyntaxTree;

Alpha [a-zA-Z_]
Digit [0-9]
AlphaDigit {Alpha}|{Digit}
INTNUM {Digit}+
ID {Alpha}{AlphaDigit}* 

%%

":=" { return (int)Tokens.ASSIGN; }
";" { return (int)Tokens.SEMICOLUMN; }
"-" { return (int)Tokens.MINUS; }
"+" { return (int)Tokens.PLUS; }
"*" { return (int)Tokens.MULT; }
"/" { return (int)Tokens.DIVIDE; }
"<" { return (int)Tokens.LT; }
">" { return (int)Tokens.GT; }
"<=" { return (int)Tokens.LE; }
">=" { return (int)Tokens.GE; }
"=" { return (int)Tokens.EQ; }
"#" { return (int)Tokens.NE; }
"(" { return (int)Tokens.LPAREN; }
")" { return (int)Tokens.RPAREN; }
"," { return (int)Tokens.COLUMN; }
"~" { return (int)Tokens.NOT; }
"&" { return (int)Tokens.AND; }
"." { return (int)Tokens.COMMA; }
":" { return (int)Tokens.COLON; }
"!" { return (int)Tokens.EXCLAMATION; }
\x01 { return (int)Tokens.INVISIBLE; }

{ID}  { 
  int res = ScannerHelper.GetIDToken(yytext);
  if (res == (int)Tokens.ID)
	yylval.sVal = yytext;
  return res;
}

{INTNUM} { 
  yylval.iVal = int.Parse(yytext); 
  return (int)Tokens.INTNUM; 
}

%{
  yylloc = new QUT.Gppg.LexLocation(tokLin, tokCol, tokELin, tokECol);
%}

%%

  public override void yyerror(string format, params object[] args) 
  {
    string [] ww = new string[args.Length-1];
	for (int i=1; i<args.Length; i++)
	  ww[i-1]=(string)args[i];
	string w = string.Join("  ",ww);  
    string errorMsg = string.Format(" :  {0},   {1}", args[0], w);
    PT.AddError(errorMsg,yylloc);
  }

class ScannerHelper 
{
  private static Dictionary<string,int> keywords;

  static ScannerHelper() 
  {
    keywords = new Dictionary<string,int>();
    keywords.Add("TRUE",(int)Tokens.TRUE);
    keywords.Add("FALSE",(int)Tokens.FALSE);
    keywords.Add("ODD",(int)Tokens.ODD);
    keywords.Add("OR",(int)Tokens.OR);
    keywords.Add("IF",(int)Tokens.IF);
    keywords.Add("THEN",(int)Tokens.THEN);
    keywords.Add("ELSE",(int)Tokens.ELSE);
    keywords.Add("BEGIN",(int)Tokens.BEGIN);
    keywords.Add("END",(int)Tokens.END);
    keywords.Add("WHILE",(int)Tokens.WHILE);
    keywords.Add("DO",(int)Tokens.DO);
    keywords.Add("BOOLEAN",(int)Tokens.BOOLEAN);
    keywords.Add("INTEGER",(int)Tokens.INTEGER);
    keywords.Add("MODULE",(int)Tokens.MODULE);
    keywords.Add("CONST",(int)Tokens.CONST);
    keywords.Add("VAR",(int)Tokens.VAR);
  }
  public static int GetIDToken(string s)
  {
	if (keywords.ContainsKey(s))
	  return keywords[s];
	else
      return (int)Tokens.ID;
  }
}
